@@ -15,16 +15,22 @@
  * TODO after change type it is not refreshed on list
  * TODO implements URL to click on
  * TODO implements copy of information field (and images?)
+ * TODO avoid invalid empty password
+ * TODO allow to delete database
+ * TODO allow to change password
  */
 
 #include "eWallet.h"
 #include "Constants.h"
 #include "DatabaseManager.h"
 #include <Qt>
+#include <QtGui>
 #include <QFile>
 #include <QDBusInterface>
 #include <QDBusConnection>
 #include <QDesktopServices>
+#include <QListView>
+
 //#ifdef Q_WS_MAEMO_5
 //#include <mce/mode-names.h>
 //#include <mce/dbus-names.h>
@@ -32,40 +38,24 @@
 
 DatabaseManager * dbManager;
 
-eWallet::eWallet(QWidget *parent) :
-		QMainWindow(parent)
+eWallet::eWallet(QWidget *parent) : QMainWindow(parent)
 {
+	this->setAttribute(Qt::WA_Maemo5StackedWindow);
+
 	ascSort = true;
 	currentFilter = "";
-	currentTarget = NULL;
 
-//qDBusRegisterMetaType<Property>();
-//qDBusRegisterMetaType< QList<Property> >();
+	//this->setAttribute(Qt::WA_Maemo5AutoOrientation, true);
+
 	loadTypesAndFields ();
 
 	::dbManager = new DatabaseManager(this);
 
 	askPassword ();
 
-//	QDBusInterface *interface = new QDBusInterface(MCE_SERVICE, MCE_REQUEST_PATH,
-//                                   MCE_REQUEST_IF, QDBusConnection::systemBus(),
-//                                   this);
-//
-//    QDBusMessage reply = interface->call(MCE_DISPLAY_SIG);
-//    if (reply.type() == QDBusMessage::ErrorMessage)
-//        qDebug() << "error? " << reply.errorMessage();
-
 	QDBusConnection conn = QDBusConnection::systemBus(); //QString(MCE_SERVICE), QString(MCE_SIGNAL_PATH)
 	bool success = conn.connect("","", QString("com.nokia.mce.signal"/*MCE_SIGNAL_IF*/), QString("display_status_ind"/*MCE_DISPLAY_SIG*/) , this, SLOT(listenInactivity(QString)));
-//	bool success = conn.connect(QString(MCE_SERVICE), QString(MCE_SIGNAL_PATH), QString(MCE_SIGNAL_IF), QString(MCE_DISPLAY_SIG) /*MCE_DISPLAY_SIG MCE_DEVLOCK_MODE_GET MCE_DISPLAY_STATUS_GET MCE_INACTIVITY_STATUS_GET*/, this, SLOT(listenInactivity(QString)));
-//	//bool success = conn.connect("com.nokia.mce","/com/nokia/mce/signal","com.nokia.mce.signal", MCE_TKLOCK_MODE_GET, this, SLOT(listenInactivity(string)));
-
-	qDebug() << "SUCCESS : " << success;
-
-	//timer = new QTimer(this);
-	//connect(timer, SIGNAL(timeout()), this, SLOT(lockWallet()));
-	//timer->start(5000);
-	//showList  ();
+	//bool success = conn.connect("","", QString(MCE_SIGNAL_IF), QString(MCE_DISPLAY_SIG) , this, SLOT(listenInactivity(QString)));
 }
 
 void eWallet::showList () {
@@ -74,16 +64,20 @@ void eWallet::showList () {
 	//                         // Central widget and Layout
 	centralWidget = new QWidget(this);
 
-	setContextMenuPolicy(Qt::DefaultContextMenu);
-	centralWidget->setContextMenuPolicy(Qt::DefaultContextMenu);
+//	setContextMenuPolicy(Qt::DefaultContextMenu);
+//	centralWidget->setContextMenuPolicy(Qt::DefaultContextMenu);
 
 	vboxlayout = new QVBoxLayout;
 	vboxlayout->setSpacing(0);
 	vboxlayout->setMargin(0);
 	// Custom list box
-	fnListWidget = new QFNListWidget(this);
+
+	fnListWidget = new QListWidget(this);
 	vboxlayout->addWidget(fnListWidget);
 
+	QObject::connect(fnListWidget, SIGNAL(itemClicked(QListWidgetItem *)), this, SLOT(selectItem(QListWidgetItem *)));
+	QObject::connect(fnListWidget, SIGNAL(itemDoubleClicked(QListWidgetItem *)), this, SLOT(selectDItem(QListWidgetItem *)));
+
 	inputWidget = new QHBoxLayout;
 	inputWidget->setContentsMargins(2, 2, 2, 0);
 	vboxlayout->addLayout(inputWidget);
@@ -114,6 +108,22 @@ void eWallet::showList () {
 	centralWidget->setFocus ();
 }
 
+void eWallet::selectItem(QListWidgetItem * item) {
+	if (fnListWidget->currentItem() != item) {
+		fnListWidget->setCurrentItem(item);
+	}
+	Target * target = list.at(fnListWidget->currentRow());
+	viewTarget(target);
+}
+
+void eWallet::selectDItem(QListWidgetItem * item) {
+	if (fnListWidget->currentItem() != item) {
+		fnListWidget->setCurrentItem(item);
+	}
+	Target * target = list.at(fnListWidget->currentRow());
+	editTarget(target);
+}
+
 void eWallet::listenInactivity(QString s) {
 	//qDebug () << "HELLO " << s;
 	if (s == QString("off") && ::dbManager->isReady ()) {
@@ -152,7 +162,21 @@ void eWallet::textChanged (const QString & text) {
 }
 
 void eWallet::filterBy (const QString & s) {
-	fnListWidget->filterBy (s);
+	for (int i = 0; i < list.size (); i++) {
+		Target * target = list.at(i);
+
+		if (s.length() == 0) {
+			fnListWidget->item(i)->setHidden(false);
+		}
+		else {
+			if (target->name.contains(s, Qt::CaseInsensitive)) {
+				fnListWidget->item(i)->setHidden(false);
+			}
+			else {
+				fnListWidget->item(i)->setHidden(true);
+			}
+		}
+	}
 }
 
 void eWallet::_cancelSearch () {
@@ -286,9 +310,6 @@ void eWallet::loadTypesAndFields () {
 	  }
 }
 
-void eWallet::timerEvent(QTimerEvent* event) {
-
-}
 void eWallet::keyPressEvent(QKeyEvent* event) {
 	//qDebug()<<"key pressed " << event->key();
 	
@@ -331,15 +352,37 @@ void eWallet::showTargets(bool reset)
 		qSort (list.begin(),list.end(),sortTargetsDES);
 
 	for (int i = 0; i < list.size (); i++) {
-		QFNListItem* listitem = new QFNListItem(list.at(i), this);
-		fnListWidget->addItem(listitem, Qt::AlignTop | Qt::AlignLeft);
+		Target * target = list.at(i);
+
+		QPixmap image;
+		if (target->getPictureSmall() != NULL) {
+			image = QPixmap::fromImage (*target->getPictureSmall());
+
+			// at this moment we don't need anymore original images!
+			target->freeResources();
+		} else {
+			QString tmp = ":/resources/empty_";
+			tmp += target->type.simplified().replace(' ', '_');
+			tmp += ".png";
+
+			if (QResource (tmp).isValid()) {
+				image = QPixmap (tmp);
+			}
+			else {
+				image = QPixmap (":/resources/empty_Others.png");
+			}
+		}
+
+		QListWidgetItem* listitem = new QListWidgetItem(image, target->name, fnListWidget);
+
+		fnListWidget->addItem(listitem);
 		
-		QObject::connect(listitem, SIGNAL(viewTarget(Target*)),
-                      this, SLOT(viewTarget(Target*)));
-		QObject::connect(listitem, SIGNAL(editTarget(Target*)),
-                      this, SLOT(editTarget(Target*)));
-		QObject::connect(listitem, SIGNAL(deleteTarget(Target*)),
-                      this, SLOT(deleteTarget(Target*)));
+//		QObject::connect(listitem, SIGNAL(viewTarget(Target*)),
+//                      this, SLOT(viewTarget(Target*)));
+//		QObject::connect(listitem, SIGNAL(editTarget(Target*)),
+//                      this, SLOT(editTarget(Target*)));
+//		QObject::connect(listitem, SIGNAL(deleteTarget(Target*)),
+//                      this, SLOT(deleteTarget(Target*)));
 	}
 
 	filterBy(currentFilter);
@@ -350,6 +393,7 @@ void eWallet::showTargets(bool reset)
 void eWallet::createMenu()
 {
 	menuBar()->addAction("New", this, SLOT(newTarget()));
+	menuBar()->addAction("Edit", this, SLOT(_editcTarget()));
 	menuBar()->addAction("Invert Sort", this, SLOT(sort()));
 	menuBar()->addAction("Import", this, SLOT(import()));
 	menuBar()->addAction("Export", this, SLOT(exportData()));
@@ -372,27 +416,59 @@ bool eWallet::_insertTarget(Target* target)
 		
 		target->freeFieldList();
 
-		QFNListItem* listitem = new QFNListItem(target, this);
-		//qDebug () << "Adding new item 2";
-		fnListWidget->addItem(listitem, Qt::AlignTop | Qt::AlignLeft);
-		
-		QObject::connect(listitem, SIGNAL(viewTarget(Target*)),
-                      this, SLOT(viewTarget(Target*)));
-		QObject::connect(listitem, SIGNAL(editTarget(Target*)),
-                      this, SLOT(editTarget(Target*)));
-		QObject::connect(listitem, SIGNAL(deleteTarget(Target*)),
-                      this, SLOT(deleteTarget(Target*)));
-					  
-		update ();
-//qDebug () << "Adding new item 4";
+		list.append(target);
+
+//		if (ascSort)
+//			qSort (list.begin(),list.end(),sortTargetsASC);
+//		else
+//			qSort (list.begin(),list.end(),sortTargetsDES);
+
+		QPixmap image;
+		if (target->getPictureSmall() != NULL) {
+			image = QPixmap::fromImage (*target->getPictureSmall());
+
+			// at this moment we don't need anymore original images!
+			target->freeResources();
+		} else {
+			QString tmp = ":/resources/empty_";
+			tmp += target->type.simplified().replace(' ', '_');
+			tmp += ".png";
+
+			if (QResource (tmp).isValid()) {
+				image = QPixmap (tmp);
+			}
+			else {
+				image = QPixmap (":/resources/empty_Others.png");
+			}
+		}
+
+		QListWidgetItem* listitem = new QListWidgetItem(image, target->name, fnListWidget);
+
+		int pos = list.indexOf(target);
+		qDebug () << "position of new " << pos << " OF " << list.size ();
+
+		if (pos < list.size()) {
+			//qDebug () << "insert position of new " << pos << " OF " << list.size ();
+			fnListWidget->insertItem(pos, listitem);
+		}
+		else {
+			//qDebug () << "append position of new " << pos << " OF " << list.size ();
+			fnListWidget->addItem(listitem);
+		}
+		fnListWidget->setCurrentItem(listitem);
 		return true;
 	}
 	return false;
 }
 bool eWallet::_deleteTarget(Target* target)
 {
+	int i = list.indexOf(target);
+	qDebug () << "DELETE ON " << i;
 	if (::dbManager->deleteTarget(target)) {
-		QFNListItem* listitem = fnListWidget->itemAt (row);
+		list.removeAt(i);
+
+		//QFNListItem* listitem = (QFNListItem*)fnListWidget->currentItem(); //itemAt (row);
+		//QListWidgetItem * listItem = fnListWidget->currentItem();
 //		//disconnect(obj, SIGNAL(destroyed(QObject*)),               this, SLOT(windowDestroyed(QObject*)));
 //		QObject::disconnect(listitem, SIGNAL(viewTarget(Target*)),
 //                      this, SLOT(viewTarget(Target*)));
@@ -401,27 +477,29 @@ bool eWallet::_deleteTarget(Target* target)
 //		QObject::disconnect(listitem, SIGNAL(deleteTarget(Target*)),
 //                      this, SLOT(deleteTarget(Target*)));
 
-		fnListWidget->removeItem (row);
-		currentTarget = NULL;
+		fnListWidget->takeItem(i); //oveItem (row);
 //		target->freeFieldList();
 //		target->freeResources();
 //		delete target;
 		// TODO delete listitem;
-		update ();
+		//update ();
 		return true;
 	}
 	return false;
 }
 bool eWallet::_updateTarget(Target* target)
 {
+	int i = list.indexOf(target);
+	qDebug () << "UPDATE ON " << i;
+
 	if (::dbManager->updateTarget(target)) {
 		::dbManager->updateFields (target);
 		
 		target->freeFieldList();
 
-		if (row >= 0) {
-			fnListWidget->itemAt (row)->setText (target->name);
-		}			
+		QListWidgetItem* listitem = fnListWidget->item (i);
+		listitem->setText(target->name);
+		//((QFNListItem*)fnListWidget->currentItem())->setText (target->name);
 		return true;
 	}
 	return false;
@@ -429,10 +507,31 @@ bool eWallet::_updateTarget(Target* target)
 
 bool eWallet::_updateTargetPicture(Target* target)
 {
+	int i = list.indexOf(target);
+	qDebug () << "UPDATE ON " << i;
 	if (::dbManager->updateTargetPicture(target)) {
-		if (row >= 0) {
-			fnListWidget->itemAt (row)->setImage (target->getPictureSmall(), target->type);
+
+		QPixmap image;
+		if (target->getPictureSmall() != NULL) {
+			image = QPixmap::fromImage (*target->getPictureSmall());
+
+			// at this moment we don't need anymore original images!
+			target->freeResources();
+		} else {
+			QString tmp = ":/resources/empty_";
+			tmp += target->type.simplified().replace(' ', '_');
+			tmp += ".png";
+
+			if (QResource (tmp).isValid()) {
+				image = QPixmap (tmp);
+			}
+			else {
+				image = QPixmap (":/resources/empty_Others.png");
+			}
 		}
+
+		QListWidgetItem* listitem = fnListWidget->item (i);
+		listitem->setIcon(image);
 		return true;
 	}
 	return false;
@@ -449,10 +548,8 @@ void eWallet::deleteTarget(Target * target) {
 
 	//qDebug()<<"delete ";
 
-	row = fnListWidget->findRow(target);
+	//row = fnListWidget->findRow(target);
 	
-	currentTarget = target;
-
 	_deleteTarget (target);
 
 	delete target;
@@ -481,9 +578,9 @@ void eWallet::windowDestroyed(QObject* obj)
 void eWallet::viewTarget(Target * rent) {
 	//qDebug()<<"view ";
 
-	row = fnListWidget->findRow(rent);
+	//row = fnListWidget->findRow(rent);
 
-	currentTarget = rent;
+	//currentTarget = rent;
 
 	dbManager->getFields(rent);
 
@@ -494,12 +591,12 @@ void eWallet::viewTarget(Target * rent) {
                       this, SLOT(editTarget(Target*)));
 		connect(detailWindow, SIGNAL(newTarget()),
                       this, SLOT(newTarget()));
-#ifdef Q_WS_MAEMO_5
-		detailWindow->setAttribute(Qt::WA_Maemo5StackedWindow);
-#endif
+//#ifdef Q_WS_MAEMO_5
+//#endif
         detailWindow->setAttribute(Qt::WA_DeleteOnClose);
         connect(detailWindow, SIGNAL(destroyed(QObject*)),
                 this, SLOT(windowDestroyed(QObject*)));
+		detailWindow->setAttribute(Qt::WA_Maemo5StackedWindow);
 	    detailWindow->show(); //owMaximized();
 	}
 }
@@ -507,18 +604,16 @@ void eWallet::viewTarget(Target * rent) {
 void eWallet::editTarget(Target * rent) {
 	//qDebug()<<"edit ";
 
-	row = fnListWidget->findRow(rent);
-
-	currentTarget = rent;
+	//row = fnListWidget->findRow(rent);
 
 	dbManager->getFields(rent);
 
 	if (detailWindowEdit.isNull()) {
 		detailWindowEdit = new DetailWindow(rent, true, this, this);
-#ifdef Q_WS_MAEMO_5
+//#ifdef Q_WS_MAEMO_5
 		detailWindowEdit->setAttribute(Qt::WA_Maemo5StackedWindow);
-#endif
-		detailWindowEdit->showMaximized();
+//#endif
+		//detailWindowEdit->showMaximized();
         detailWindowEdit->setAttribute(Qt::WA_DeleteOnClose);
         connect(detailWindowEdit, SIGNAL(destroyed(QObject*)),
                 this, SLOT(windowDestroyed(QObject*)));
@@ -636,15 +731,12 @@ void eWallet::import() {
 void eWallet::newTarget() {
 	//qDebug()<<"new ";
 
-	row = -1;
-	currentTarget = NULL;
-
 	if (detailWindowEdit.isNull()) {
 		detailWindowEdit = new DetailWindow(NULL, true, this, this);
-#ifdef Q_WS_MAEMO_5
+//#ifdef Q_WS_MAEMO_5
 		detailWindowEdit->setAttribute(Qt::WA_Maemo5StackedWindow);
-#endif
-		detailWindowEdit->showMaximized();
+//#endif
+		//detailWindowEdit->showMaximized();
         detailWindowEdit->setAttribute(Qt::WA_DeleteOnClose);
         connect(detailWindowEdit, SIGNAL(destroyed(QObject*)),
                 this, SLOT(windowDestroyed(QObject*)));
@@ -667,29 +759,6 @@ void eWallet::contextMenuEvent (QContextMenuEvent * event) {
 	}
 }
 */
-/*
-void eWallet::editTarget(Target * rent)
-{
-	row = fnListWidget->findRow(rent);
-
-	qDebug()<<"Edit ";
-
-	if (createTargetDlg.isNull()) {
-			createTargetDlg = new CreateTargetDialog(rent, this);
-#if defined(Q_OS_WIN32)
-			createTargetDlg->setFixedSize(640,480);
-#elif defined(Q_WS_HILDON)
-			createTargetDlg->setMinimumSize(size().width(),size().height()-100);
-#else
-			createTargetDlg->showMaximized();
-#endif
-			createTargetDlg->exec();
-			delete createTargetDlg;
-			createTargetDlg = 0;
-		
-	}
-}
-*/
 
 void eWallet::exportData() {
 	QString path(QDesktopServices::storageLocation(QDesktopServices::HomeLocation));
@@ -729,14 +798,23 @@ void eWallet::exportData() {
 		  }
 	}
 }
+
 void eWallet::_editcTarget() {
-	editTarget(this->fnListWidget->itemAt(row)->getSource());
+	if (fnListWidget->currentRow() >= 0) {
+		Target * target = list.at(fnListWidget->currentRow());
+		editTarget(target);
+	}
+	//editTarget(((QFNListItem*)fnListWidget->currentItem())->getSource());
 }
 void eWallet::_deletecTarget() {
-	deleteTarget(this->fnListWidget->itemAt(row)->getSource());
+	//deleteTarget(((QFNListItem*)fnListWidget->currentItem())->getSource());
 }
 void eWallet::_viewcTarget() {
-	viewTarget(this->fnListWidget->itemAt(row)->getSource());
+	if (fnListWidget->currentRow() >= 0) {
+		Target * target = list.at(fnListWidget->currentRow());
+		viewTarget(target);
+	}
+	//viewTarget(((QFNListItem*)fnListWidget->currentItem())->getSource());
 }
 
 //eWallet closeEvent function:
